;
; GG Nibbles
; Release 4
; 04/2000
; Source code
;
; This is the source code of my first Game Gear game.
;
; This source code isn't completly optimised and commented yet.
; Please let me know which parts of it you don't understand.
; Let me also know if you find it useful.
;
; The files with .INC extension are include files and belong to this code.
; The files with no extension are data files (tiledata and palettes).
; The file called levels.dat contains the descriptions of each level.
;
; If you have any questions or comments or if you found any bugs,
; contact me at: mkon@gmx.de
;
; Compile it using the Table Assembler with:
;   tasm NIBBLES.ASM -80 -x -b -f00
;

.org $0000

; Constants
TRUE  = -1
FALSE = 0

RAM_START           = $C000
VRAM_TILES          = $4000
VRAM_TILEMAP        = $7800
VRAM_SPRITES        = $7F00
VRAM_TILE_PALETTE   = $C000
VRAM_SPRITE_PALETTE = $C020
STOP_SPRITES        = 208   ; special code to stop the sprites

; Game
SCORE_X          = 110      ; the position of the score
SCORE_Y          = 25

MAX_LEVEL        = 23       ; the last level
FOOD_TO_COLLECT  = 10       ; defines how many food you have to collect
AMOUNT_OF_FOOD   = 9        ; defines how many different food tiles exist
MINIMAL_SPEED    = 4        ; the slowest speed that is possible

TAIL_MAXLENGTH   = 39       ; the maximum length of the tail
MAX_X            = 19       ; the size of the playfield (0..19, 0..17)
MAX_Y            = 17
MAX_HEAD_ANIM    = 26       ; used for the head animation;
                            ; it defines the length of the HEAD_ANIM_TABLE

DIRECTION_RIGHT  = 1
DIRECTION_LEFT   = 2
DIRECTION_UP     = 3
DIRECTION_DOWN   = 0
NO_DIRECTION     = 4        ; in this case, the snake doesn't move
                            ; (only at the start of the level)

; Control
P1_CONTROL_PORT = $DC
P1_UP    = 1
P1_DOWN  = 2
P1_LEFT  = 4
P1_RIGHT = 8
P1_B1    = 16
P1_B2    = 32

GG_START_BUTTON_PORT = $00
GG_START_BUTTON      = 128

; GFX
KONRAD_LOGO_XSIZE    = 10
KONRAD_LOGO_YSIZE    = 3
KONRAD_LOGO_POSITION = $7802+660
TITLE_XSIZE          = 11
TITLE_YSIZE          = 5
TITLE_POSITION       = $790C+200
TITLE_SIZE           = TITLE_XSIZE * TITLE_YSIZE * 32
THE_END_XSIZE        = 11
THE_END_YSIZE        = 3
THE_END_POSITION     = $790C+136
THE_END_SIZE         = THE_END_XSIZE * THE_END_YSIZE * 32
OPTIONS_GFX_SIZE     = 1152
GAMEGFX_SIZE         = 2560
GFX_HEAD             = 0
GFX_COL_HEAD         = 63
GFX_BODY             = 20           ; There are 12 * 2 different body-types.
                                    ; The first 12 are just the normal body
                                    ; tiles, while the second 12 are drawn
                                    ; at the end of the snake.
GFX_NUMBERS          = 44
GFX_FOOD             = 54
GFX_WALLS            = 67

; Game variables
; Note that some of these are just placed somewhere in RAM.
; There's no special order.
START_BUTTON_STILL_PRESSED = RAM_START
START_BUTTON_STATUS        = RAM_START+1
OLD_JOY_STATUS             = RAM_START+2047

STARTLEVEL                 = RAM_START+2044 ; these two variables can be
SPEED                      = RAM_START+2045 ; set in the option screen

SLOWDOWN_COUNTER            = RAM_START+2    ; this is for slowing
                                             ; down the game
HEAD_ANIM                   = RAM_START+3
CLEAR_LAST_TAILPART_COUNTER = RAM_START+4    ; if this variable gets 0, the
                                             ; last tile must be cleared

RANDOM                      = RAM_START+5    ; the first random number
                                             ; (used for the x position of
                                             ;  the new food)
RANDOM2                     = RAM_START+2000 ; the second random number
                                             ; (used for the y position of
                                             ;  the new food)
RANDOM3                     = RAM_START+2001 ; the third random number
                                             ; (used for choosing a
                                             ;  new piece of food)

LEVEL_NUMBER                = RAM_START+6    ; starts from 0
FOOD_COLLECTED              = RAM_START+7
COLLISION                   = RAM_START+8    ; this is TRUE if the
                                             ; player has collided

START_X                     = RAM_START+9    ; the start positions
START_Y                     = RAM_START+10
SCORE                       = RAM_START+2048 ; this is an array of 4 bytes,
                                             ; because the score conists of
                                             ; 4 numbers that are used almost
                                             ; like BCD-values

TAIL_LENGTH     = RAM_START+11  ; the length of the tail
LAST_TAILPART   = RAM_START+12  ; the index of the last tailpart and
LAST_TAILPART_X = RAM_START+13  ; its positions
LAST_TAILPART_Y = RAM_START+14
POSITION_X      = RAM_START+15  ; the newest position
POSITION_Y      = RAM_START+16
PR_POSITION_X   = RAM_START+17  ; the previous position
PR_POSITION_Y   = RAM_START+18
TAILPART_X      = RAM_START+256 ; an array that holds the positions
TAILPART_Y      = RAM_START+512 ; of each tailpart

DIRECTION       = RAM_START+19  ; the direction
PR_DIRECTION    = RAM_START+20  ; the direction of the last move

; Dummies
DUMMY_PALETTE = RAM_START+21
DUMMY         = RAM_START+2046
DUMMY2        = RAM_START+2042
DUMMY3        = RAM_START+2043
DUMMY_TILES   = RAM_START+4096





;--- STARTUP CODE -----------------------------------------------------------
  di             ; Disable Interrupts
  im 1           ; Interrupt Mode 1
  ld sp, $dff0   ; set the stack pointer
  jp START
  .byte  "GG",32,"Nibbles",32,
  .byte  "by Martin Konrad (mkon@gmx.de)"
;----------------------------------------------------------------------------
NMI:
.org $66
  retn
  .byte  32,"4/2000",32,32,32,"Third release"
;----------------------------------------------------------------------------
START:
;--- INITIALIZATION ---------------------------------------------------------
  call SETUP_FRAMES          ; setup the frame registers
    ld hl,VDP_REGS_TABLE
  call SET_VDP_REGS_TABLE    ; set the vdp registers
                             ; (the screen is disabled in this
                             ;  table and has to be enabled by
                             ;  using ENABLE_SCREEN)
  call CLEAR_TILEMAP         ; clear VRAM and...
  call CLEAR_TILES
    ld a, 0
  call DISABLE_SPRITES       ; ... stop the sprites
;--- PROGRAM ----------------------------------------------------------------

  call LOGO                  ; display the Konrad-logo
  call TITLE                 ; display the title
  call OPTIONS               ; the option screen

  ld a, 0                    ; reset score
  ld (SCORE), a
  ld (SCORE+1), a
  ld (SCORE+2), a
  ld (SCORE+3), a

  ld a, (STARTLEVEL)         ; set the level to start with
  ld (LEVEL_NUMBER), a
PLAY:
  call GAME                  ; play the game
    ld a, (COLLISION)        ; check if the player has collided
    cp FALSE                 ; if no, play the next level
    jr Z, NEXT_LEVEL                 
      ld c, 2                 ; if yes, subtract 150 points
      ld d, 1                 ; from the score (1 * 10^2 + 5 * 10^1 = 150)...
    call SUBTRACT_FROM_SCORE
      ld c, 1
      ld d, 5
    call SUBTRACT_FROM_SCORE
    jr PLAY                   ; ... and play the level again
NEXT_LEVEL:
    ld hl, LEVEL_NUMBER      ; increase the LEVEL_NUMBER
    inc (HL)
    ld a, (HL)
    cp MAX_LEVEL+1           ; if the last level has been reached,
    call Z, THE_END          ; jump to the end
    jr PLAY                  ; if not, play the next level

;============================================================================
;=                                                                          =
;= G A M E                                                                  =
;=                                                                          =
;============================================================================
; About the game:
;   At the beginning the snake is placed at the start position that
;   is defined in the level descriptions. All used tailparts are
;   initiated at this position too. The last tailpart is attached
;   at the new position of the snake every move.
GAME:
  call LOAD_GAMEGFX             ; load the GAMEGFX
  call DRAW_LEVEL               ; draw the current level
  call CLEAR_PALETTE            ; clear the palette
                                ; (it will be faded in later...)
  call INITIATE_GAME_VARIABLES  ; initiate the required game variables

; Game start
  call DRAW_HEAD                ; draw the head, so that it will
                                ; allready displayed in the first frame
  call SET_NEW_FOOD             ; set the first food

    ld a, 4                     ; disable all sprites after 3 because the
  call DISABLE_SPRITES          ; first 4 sprites (0..3) are used to
                                ; display the score
  call SET_SCORE_SPRITES        ; draw the score using sprites

  call ENABLE_SCREEN            ; now enable the screen and

  call GET_START_BUTTON_STATUS  ; get the latest status of the start button
GAME_LOOP:
    ld d, 1                        ; wait for vertical sync
  call VWAIT

    ld ix, GAME_PALETTE
    ld b, 5
  call FADE_IN_ONE_STEP

  call CHANGE_HEAD_ANIM            ; animate the head
    call DRAW_HEAD
  call GET_DIRECTION               ; get the player direction -
                                   ; for a better control, this
                                   ; routine isn't only called
                                   ; when moving the snake

  call CHECK_PAUSE                 ; Check for pause

  ld hl, SLOWDOWN_COUNTER          ; decrease the SLOWDOWN_COUNTER
  dec (HL)                         ; until it reaches 0 and
  jr NZ, GAME_LOOP                 ; the snake moves

  call MOVE_PLAYER                 ; move the snake
  call CHECK_COLLISION             ; check collisions
  call DRAW_BODY                   ; draw the body at the previous position
  call DRAW_LAST_TAILPART          ; draw the end of the snake,
                                   ; which is the last tailpart.
  ld a, (DIRECTION)                ; save the direction
  ld (PR_DIRECTION), a             ; to the previous direction

  call DRAW_HEAD                   ; draw the head at the new position

  ld hl, SLOWDOWN_COUNTER          ; reset the SLOWDOWN_COUNTER
  ld a, (SPEED)
  ld (HL), a

  ld a, (COLLISION)                ; check if the player has collided
  cp TRUE
  jr Z, EXIT_GAME                  ; if yes, then exit the game

  ld a, (FOOD_COLLECTED)           ; check if enough food
  cp FOOD_TO_COLLECT               ; has been collected
  jr Z, EXIT_GAME                  ; if yes, then exit the game

  jr GAME_LOOP                     ; jump back

EXIT_GAME:
  ld a, (COLLISION)              ; check if the player has collided
  cp FALSE                       ; if not,
  jr Z, NO_COLLISION_HAPPENED    ; jump away
    ld a, (POSITION_Y)           ; take the current position, ...
    ld b, a
    ld a, (POSITION_X)
  call CALCULATE_SCREEN_ADDRESS  ; calculate its screen address and...
    ld a, (DIRECTION)
    add a, GFX_COL_HEAD          ; ... (take the collided head animation) ...
    ld b, a
    ld d, 0
  call DRAW_TILE                 ; ... draw the collided head

NO_COLLISION_HAPPENED:
    ld a, 0                        ; fade out from 0 (normal)
    ld d, 7                        ;    with 7 steps and
    ld e, 6                        ;    a slowdown of 6 VWAITs
    ld ix, GAME_PALETTE            ;    using the GAME_PALETTE
  call FADE_OUT                    ;    to -7 (fully dark)

  call DISABLE_SCREEN              ; now disable the screen
    ld a, 0                        ; and stop the sprites that
  call DISABLE_SPRITES             ; have been used for displaying the score
  ret

;============================================================================
;=                                                                          =
;= G A M E   S U B F U N C T I O N S                                        =
;=                                                                          =
;----------------------------------------------------------------------------

;--- ADD TO SCORE -----------------------------------------------------------
; This procedure adds a value to the score, which
; uses some kind of BCD-values that can be easily displayed.
;--- D = Value   C = Number of value ( 10^C ) -------------------------------
ADD_TO_SCORE:
  ld hl, SCORE
  ld b, 0
  add hl, bc
  ld a, 4
  sub c
  ld e, a
  ld a, (HL)
  add a, d
ADD_TO_NEXT:
  cp 10
  jp C, DONT_ADD_TO_NEXT
  sub 10
  ld (HL), a
  inc hl

  ld a, e
  cp 1
  ret Z

  inc (HL)
  ld a, (HL)
  dec e
  jr nz, ADD_TO_NEXT
DONT_ADD_TO_NEXT:
  ld (HL), a
  ret
;--- SUBTRACT FROM SCORE ----------------------------------------------------
; This procedure subtracts a value from the score.
;--- D = Value   C = Number of value ( 10^C ) -------------------------------
SUBTRACT_FROM_SCORE:
  ld hl, SCORE
  ld b, 0
  add hl, bc
  ld a, 4
  sub c
  ld e, a
  ld a, (HL)
  sub d
SUBTRACT_FROM_NEXT:
  jr NC, DONT_SUBTRACT_FROM_NEXT
  push af
  ld a, e
  cp 1
  jr Z, MAKE_IT_ZERO
  pop af
  add a, 10
  ld (HL), a
  inc hl
  dec (HL)
  ld a, (HL)
  dec e
  jr Z, DONT_SUBTRACT_FROM_NEXT
  inc a
  sub 1
  jr SUBTRACT_FROM_NEXT
DONT_SUBTRACT_FROM_NEXT:
  ld (HL), a
  ret
MAKE_IT_ZERO:
  pop af
  ld a, 0
  ld (SCORE), a
  ld (SCORE+1), a
  ld (SCORE+2), a
  ld (SCORE+3), a
  ret
;--- SET SCORE SPRITES ------------------------------------------------------
SET_SCORE_SPRITES:
  ld b, 4
  ld c, 0
  ld d, SCORE_X+27
  ld hl, SCORE
SET_SPRITES_LOOP:
  push bc
  push hl
    ld b, GFX_NUMBERS
    ld a, (HL)
    add a, b
    ld e, SCORE_Y
  call SET_SPRITE
  ld a, d
  sub 9
  ld d, a
  pop hl
  pop bc
  inc hl
  inc c
  dec b
  jr NZ, SET_SPRITES_LOOP
  ret
;--- SET NEW FOOD -----------------------------------------------------------
SET_NEW_FOOD:
  ld a, (FOOD_COLLECTED)
  cp (FOOD_TO_COLLECT)
  ret Z

    ld b, MAX_X+1              ; get a random number from 0 to MAX_X
  call GET_RANDOM
  ld d, a
    ld b, MAX_Y+1              ; get a random number from 0 to MAX_Y
  call GET_RANDOM2             ; (using the second randomizer)

  ld e, a
  push de

    ld a, d                     ; calculate the screen address
    ld b, e                     ; of the new food
  call CALCULATE_SCREEN_ADDRESS
  pop de

  push hl                       ; save HL
  call READ_TILE                ; look if there's...
  pop hl                        ; get HL back

  cp GFX_WALLS                  ; ... a tile <> GFX_WALLS (<- clear tile)
  jr NZ, SET_NEW_FOOD           ; if yes, try again

      ld b, AMOUNT_OF_FOOD      ; get a random number between 0
    call GET_RANDOM3            ; and the AMOUNT_OF_FOOD - 1
                                ; (using the second randomizer)

    add a, GFX_FOOD
    ld b, a
    ld d, 0
  call DRAW_TILE
  ret
;--- GET RANDOM NUMBER ------------------------------------------------------
; This is a very primitive randomizer.
; It just adds 5 to the random number every time.
;--- B = max number + 1 -----------------------------------------------------
GET_RANDOM:
  ld a, (RANDOM)
  add a, 5                ; add 5
  ld (RANDOM), a
  cp b                    ; if it's out of range,
  jr NC, GET_RANDOM       ; calculate again

  ld a, (RANDOM)
  inc a
  ld (RANDOM), a
  ret
;--- GET RANDOM NUMBER (2) --------------------------------------------------
; This one is almost like the one above.
; It seems better to have different
; randomizers for different things.
;--- B = max number + 1 -----------------------------------------------------
GET_RANDOM2:
  ld a, (RANDOM2)
  add a, 5                 ; add 5
  ld (RANDOM2), a
  cp b                     ; if it's out of range,
  jr NC, GET_RANDOM2       ; calculate again
  ret
;--- GET RANDOM NUMBER (3) --------------------------------------------------
; This one is almost like the one above.
; But it adds only 3.
;--- B = max number + 1 -----------------------------------------------------
GET_RANDOM3:
  ld a, (RANDOM3)
  add a, 3                 ; add 3
  ld (RANDOM3), a
  cp b                     ; if it's out of range,
  jr NC, GET_RANDOM3       ; calculate again
  ret
;--- INITIATE GAME VARIABLES ------------------------------------------------
INITIATE_GAME_VARIABLES:
; Set Tail length
  ld a, (LEVEL_NUMBER)           ; the tail length is
  add a, a                       ; (LEVEL_NUMBER) * 2
  inc a                          ; + 1
  cp TAIL_MAXLENGTH              ; shorten the snake
  jp C, NOT_MAXLENGTH            ; if its length is more
  ld a, TAIL_MAXLENGTH           ; then the maximum
NOT_MAXLENGTH:
  ld (TAIL_LENGTH), a
; Set tailpart X positions
    ld hl, TAILPART_X
    ld b, TAIL_MAXLENGTH
    inc b
    ld a, (START_X)
  call FILL_MEM
; Set tailpart Y positions
    ld hl, TAILPART_Y
    ld b, TAIL_MAXLENGTH
    inc b
    ld a, (START_Y)
  call FILL_MEM
; Set other positions
  ld a, (START_X)
  ld (POSITION_X), a
  ld (PR_POSITION_X), a
  ld (LAST_TAILPART_X), a
  ld a, (START_Y)
  ld (POSITION_Y), a
  ld (PR_POSITION_Y), a
  ld (LAST_TAILPART_Y), a
; Set the SLOWDOWN_COUNTER
  ld a, (SPEED)
  ld (SLOWDOWN_COUNTER), a
; Set clear_last_tailpart_counter
  ld a, (TAIL_LENGTH)
  inc a
  ld (CLEAR_LAST_TAILPART_COUNTER), a
; Set direction
  ld a, NO_DIRECTION
  ld (DIRECTION), a
  ld (PR_DIRECTION), a
; Clear variables
  ld a, 0
  ld (HEAD_ANIM), a
  ld (COLLISION), a
  ld (FOOD_COLLECTED), a
  ld (LAST_TAILPART), a
  ld (DUMMY3), a                 ; used for slowing down the fading
  ld a, -7
  ld (DUMMY), a                  ; used for the fading brightness value
  ret
;--- DRAW LAST TAILPART -----------------------------------------------------
; This procedure takes the body tile at the last position,
; adds 12 to the tile number (there are 12 * 2 body tiles)
; and displays this tile. Some normal body tiles (the first
; 12 look the same but they have to be there so that the
; right tile at the end of the snake is shown.
DRAW_LAST_TAILPART:
    ld a, (LAST_TAILPART_Y)         ; get the position of
    ld b, a                         ; the last tailpart and...
    ld a, (LAST_TAILPART_X)
  call CALCULATE_SCREEN_ADDRESS     ; ... calculate it's screen address
  call READ_TILE
  push af
    ld a, (LAST_TAILPART_Y)
    ld b, a
    ld a, (LAST_TAILPART_X)
  call CALCULATE_SCREEN_ADDRESS
    pop af
    ld d, a

    cp GFX_BODY+12                   ; don't draw if
    jp NC, DONT_DRAW_IT              ; there's already an
                                     ; tail-end tile.

    add a, 12
    ld b, a
    ld d, 0
  call DRAW_TILE
DONT_DRAW_IT:
  ret
;--- GET START BUTTON STATUS ------------------------------------------------
; This procedure looks if the start button has been pressed again so
; that the pause works correct.
GET_START_BUTTON_STATUS:
  in a, (GG_START_BUTTON_PORT)
   and GG_START_BUTTON
    jr NZ, NO_START_BUTTON_PRESSED
  ld a, (START_BUTTON_STILL_PRESSED)
  cp 1
  jr Z, NO_START_BUTTON_PRESSED_AGAIN
  ld a, 1
  ld (START_BUTTON_STATUS), a
  ld (START_BUTTON_STILL_PRESSED), a
  ret
NO_START_BUTTON_PRESSED_AGAIN:
  ld a, 0
  ld (START_BUTTON_STATUS), a
  ret
NO_START_BUTTON_PRESSED:
  ld a, 0
  ld (START_BUTTON_STATUS), a
  ld (START_BUTTON_STILL_PRESSED), a
  ret
;--- CHECK PAUSE ------------------------------------------------------------
CHECK_PAUSE:
  call GET_START_BUTTON_STATUS

    ld a, (START_BUTTON_STATUS)
    cp 1
    jr NZ, NO_PAUSE

    ld a, -1                            ; Fade out
    ld d, 3
    ld e, 2
    ld ix, GAME_PALETTE
  call FADE_OUT
  call GETST
  call GET_START_BUTTON_STATUS

    ld a, -2                            ; fade in
    ld d, 3
    ld e, 2
    ld ix, GAME_PALETTE
  call FADE_IN
NO_PAUSE:
  ret
;--- CHECK COLLISION --------------------------------------------------------
CHECK_COLLISION:
  ld a, (DIRECTION)           ; return, if the snake isn't moved
  cp NO_DIRECTION
  ret Z

    ld a, (POSITION_Y)          ; take the newest position
    ld b, a                     ; (the head hasn't been
    ld a, (POSITION_X)          ;  drawn there yet) 
  call CALCULATE_SCREEN_ADDRESS ; and calculate its screen address
  call READ_TILE                ; now read the tile
  cp GFX_WALLS                  ; if the tile on the screen is GFX_WALLS,
  ret Z                         ; then return (only the tiles with a number
                                ; of > GFX_WALLS are walls)

                                ; check if there's a food tile
  cp GFX_FOOD                   ; if the tile is less than GFX_FOOD,
  jr C, CRASH                   ; then crash
  cp GFX_FOOD + AMOUNT_OF_FOOD  ; if the tile is between GFX_FOOD and
  jr C, COLLECT_FOOD            ; GFX_FOOD + AMOUNT_OF_FOOD, then it's food

CRASH:
  ld a, TRUE
  ld (COLLISION), a

  ld a, (PR_POSITION_X)     ; set the newest positions to the previous
  ld (POSITION_X), a        ; positions so that the head won't be drawn
  ld a, (PR_POSITION_Y)     ; on a wall
  ld (POSITION_Y), a
  ret

COLLECT_FOOD:
      ld d, 5               ; add 5 points ( 5 * 10^0 )
      ld c, 0
    call ADD_TO_SCORE
      ld d, 1               ; add 10 points ( 1 * 10^1 )
      ld c, 1
    call ADD_TO_SCORE
  call SET_SCORE_SPRITES    ; display the score

  ld hl, FOOD_COLLECTED     ; increase the FOOD_COLLECTED counter
  inc (HL)

  ld hl, TAIL_LENGTH        ; save the tail length
  ld a, (HL)                ; in A
  cp TAIL_MAXLENGTH         ; don't grow anymore if the maximum
  jr Z, DONT_GROW           ; length has been already reached

    ld ix, TAILPART_X       ; insert a new tailpart x position
    ld b, 0                 ; by copying these one byte to the right
    ld a, (LAST_TAILPART)   ; at the last tailpart
    ld c, a                 ; using its position so
    add ix, bc              ; that the new tailpart
    ld hl, TAILPART_X       ; is attached at the end
    add hl, bc              ; of the snake
    inc hl
    ld a, (TAIL_LENGTH)     ; copy (TAIL_LENGTH) - (LAST_TAILPART) + 1 bytes
    ld iy, LAST_TAILPART
    sub (IY)
    ld c, a
    inc c                   ; copy one more byte because the
                            ; taillength begins at 0
  call COPY_MEM_BACKWARDS   ; it has to be copied backwars because
                            ; the memory chunks overlap

    ld ix, TAILPART_Y       ; insert a new tailpart y position
    ld b, 0
    ld a, (LAST_TAILPART)
    ld c, a
    add ix, bc
    ld hl, TAILPART_Y
    add hl, bc
    inc hl
    ld a, (TAIL_LENGTH)
    ld iy, LAST_TAILPART
    sub (IY)
    ld c, a
    inc c
  call COPY_MEM_BACKWARDS

  ld hl, TAIL_LENGTH        ; increase the taillength
  inc (HL)

  ld hl, CLEAR_LAST_TAILPART_COUNTER     ; increase it, because
  inc (HL)                               ; there's a new tailpart at
                                         ; the end of the snake now

DONT_GROW:
  call SET_NEW_FOOD         ; of course the snake is still hungry :)
  ret
;--- MOVE PLAYER ------------------------------------------------------------
MOVE_PLAYER:
  ld a, (DIRECTION)

  cp DIRECTION_RIGHT
  jr NZ, DONT_MOVE_RIGHT
  call MOVE_RIGHT
  call CHANGE_LAST_TAILPART
  ret
DONT_MOVE_RIGHT:
  ld a, (DIRECTION)
  cp DIRECTION_LEFT
  jr NZ, DONT_MOVE_LEFT
  call MOVE_LEFT
  call CHANGE_LAST_TAILPART
  ret
DONT_MOVE_LEFT:
  ld a, (DIRECTION)
  cp DIRECTION_UP
  jr NZ, DONT_MOVE_UP
  call MOVE_UP
  call CHANGE_LAST_TAILPART
  ret
DONT_MOVE_UP:
  ld a, (DIRECTION)
  cp DIRECTION_DOWN
  jr NZ, DONT_MOVE_DOWN
  call MOVE_DOWN
  call CHANGE_LAST_TAILPART
DONT_MOVE_DOWN:
  ret
;--- GET DIRECTION ----------------------------------------------------------
; If you press right and down for example, the snake would go down
; because the setting of this direction is after the setting of the
; right direction. That's why I changed it a bit and if the previous
; direction is already down, the direction won't be set to it again, so
; the direction will stay right and then flip between right and down.
; This is not a real diagonal direction but better than if the snake
; would just go down.
; The only remaining problem was that if you release the right button and
; only press the down button and the direction was already set to right
; for the next move it couldn't be set to down again. That's why I have
; included some code that checks if only up or only down is pressed.
GET_DIRECTION:
  in a, (P1_CONTROL_PORT)   ; Get the joystick status and
  ld b, a                   ; save it to B

  ld a, b                   ; check if only up or only down is pressed
  cp $FD
  jr NZ, NOT_ONLY_DOWN
  ld a, (PR_DIRECTION)      ; don't go down if the snake is
  cp DIRECTION_UP           ; already going up
  jr Z, NOT_ONLY_DOWN
  ld a, DIRECTION_DOWN
  ld (DIRECTION), a
  ret
NOT_ONLY_DOWN:
  ld a, b
  cp $FE
  jr NZ, NOT_ONLY_UP
  ld a, (PR_DIRECTION)      ; don't go up if the snake is
  cp DIRECTION_DOWN         ; already going down
  jr Z, NOT_ONLY_UP
  ld a, DIRECTION_UP
  ld (DIRECTION), a
  ret
NOT_ONLY_UP:

  ld a, b                   ; choose the direction
  and P1_RIGHT
  call Z, DIR_RIGHT
  ld a, b
  and P1_LEFT
  call Z, DIR_LEFT
  ld a, b
  and P1_UP
  call Z, DIR_UP
  ld a, b
  and P1_DOWN
  call Z, DIR_DOWN
  ret

DIR_RIGHT:
  ld a, (PR_DIRECTION)        ; don't go right if the snake is
  cp DIRECTION_LEFT           ; already going left
  jr Z, NOT_RIGHT
  ld a, DIRECTION_RIGHT
  ld (DIRECTION), a
  ret
NOT_RIGHT:
  ret

DIR_LEFT:
  ld a, (PR_DIRECTION)        ; don't go left if the snake is
  cp DIRECTION_RIGHT          ; already going right
  jr Z, NOT_LEFT
  ld a, DIRECTION_LEFT
  ld (DIRECTION), a
  ret
NOT_LEFT:
  ret

DIR_UP:
  ld a, (PR_DIRECTION)        ; don't go up if the snake is
  cp DIRECTION_DOWN           ; already going down
  jr Z, NOT_UP
  ld a, (PR_DIRECTION)        ; don't choose up again when
  cp DIRECTION_UP             ; it's already active
  jr Z, NOT_UP
  ld a, DIRECTION_UP
  ld (DIRECTION), a
  ret
NOT_UP:
  ret

DIR_DOWN:
  ld a, (PR_DIRECTION)        ; don't go down if the snake is
  cp DIRECTION_UP             ; already going up
  jr Z, NOT_DOWN
  ld a, (PR_DIRECTION)        ; don't choose down again when
  cp DIRECTION_DOWN           ; it's already active
  jr Z, NOT_DOWN
  ld a, DIRECTION_DOWN
  ld (DIRECTION), a
  ret
NOT_DOWN:
  ret
;--- CHANGE LAST TAILPART ---------------------------------------------------
; This procedure takes the next last tailpart which will be attached
; at the beginning of the snake then.
CHANGE_LAST_TAILPART:
  ld hl, LAST_TAILPART
  ld a, (HL)
  ld ix, TAIL_LENGTH
  cp (IX)
  jr NZ, DONT_RESET
  ld a, 0
  ld (HL), a
  jr SET_LAST_TAILPART_POSITION
DONT_RESET:
  ld hl, LAST_TAILPART
  inc (HL)
SET_LAST_TAILPART_POSITION:
  ld hl, TAILPART_X
  ld b, 0
  ld a, (LAST_TAILPART)
  ld c, a
  add hl, bc
  ld a, (HL)
  ld hl, LAST_TAILPART_X
  ld (HL), a

  ld hl, TAILPART_Y
  ld b, 0
  ld a, (LAST_TAILPART)
  ld c, a
  add hl, bc
  ld a, (HL)
  ld hl, LAST_TAILPART_Y
  ld (HL), a
  ret
;--- CLEAR LAST TAILPART ----------------------------------------------------
; This procedure decreases the CLEAR_LAST_TAILPART_COUNTER.
; If it gets 0 it means that there are no more tailparts at
; the last position and the last tile can be cleared.
CLEAR_LAST_TAILPART:
  ld hl, CLEAR_LAST_TAILPART_COUNTER
  dec (HL)                            ; decrease it
  ret NZ                              ; if it's still > 0, then exit
  ld a, 1                             ; set it to 1 again
  ld (HL), a
      ld a, (LAST_TAILPART_Y)         ; take the position
      ld b, a                         ; of the last tailpart
      ld a, (LAST_TAILPART_X)              
    call CALCULATE_SCREEN_ADDRESS     ; and calculate its position
    ld b, GFX_WALLS                   ; the tile called GFX_WALLS is clear
    ld d, 0                           ; the attribute is 0 as usual
  call DRAW_TILE                      ; now draw it
  ret
;--- DRAW BODY --------------------------------------------------------------
; The body tile is drawn at the previous position
DRAW_BODY:
    ld a, (PR_POSITION_Y)
    ld b, a
    ld a, (PR_POSITION_X)
  call CALCULATE_SCREEN_ADDRESS
  ld b, GFX_BODY+2

  ld a, (DIRECTION)
  cp DIRECTION_UP
  jr NZ, CHECK_NEXT
  ld b, GFX_BODY+3
  ld a, (PR_DIRECTION)
  cp DIRECTION_RIGHT
  jr NZ, NOT_RIGHT_UP_ANIM
  ld b, GFX_BODY+11
  jr DRAW
NOT_RIGHT_UP_ANIM:
  cp DIRECTION_LEFT
  jr NZ, DRAW
  ld b, GFX_BODY+10
  jr DRAW
 
CHECK_NEXT:
  ld a, (DIRECTION)
  cp DIRECTION_DOWN
  jr NZ, CHECK_NEXT2
  ld b, GFX_BODY+2
  ld a, (PR_DIRECTION)
  cp DIRECTION_RIGHT
  jr NZ, NOT_RIGHT_DOWN_ANIM
  ld b, GFX_BODY+9
  jr DRAW
NOT_RIGHT_DOWN_ANIM:
  cp DIRECTION_LEFT
  jr NZ, DRAW
  ld b, GFX_BODY+8
  jr DRAW

CHECK_NEXT2:
  ld a, (DIRECTION)
  cp DIRECTION_RIGHT
  jr NZ, CHECK_NEXT3
  ld b, GFX_BODY
  ld a, (PR_DIRECTION)
  cp DIRECTION_UP
  jr NZ, NOT_UP_RIGHT_ANIM
  ld b, GFX_BODY+4
  jr DRAW
NOT_UP_RIGHT_ANIM:
  cp DIRECTION_DOWN
  jr NZ, DRAW
  ld b, GFX_BODY+6
  jr DRAW

CHECK_NEXT3:
  ld a, (DIRECTION)
  cp DIRECTION_LEFT
  jr NZ, DRAW
  ld b, GFX_BODY+1
  ld a, (PR_DIRECTION)
  cp DIRECTION_UP
  jr NZ, NOT_UP_LEFT_ANIM
  ld b, GFX_BODY+5
  jr DRAW
NOT_UP_LEFT_ANIM:
  cp DIRECTION_DOWN
  jr NZ, DRAW
  ld b, GFX_BODY+7
  jr DRAW

DRAW:
  ld d, 0
  call DRAW_TILE
  ret
;--- CHANGE HEAD ANIM -------------------------------------------------------
; This procedure changes the head anim by taking
; the tile with the number of (HEAD_ANIM_TABLE + HEAD_ANIM) + GFX_HEAD
CHANGE_HEAD_ANIM:
  ld hl, HEAD_ANIM
  inc (HL)                       ; next head anim
  ld a, (HL)
  cp MAX_HEAD_ANIM               ; if HEAD_ANIM < MAX_HEAD_ANIM,
  ret NZ                         ; then exit,
  ld a, 0                        ; else reset HEAD_ANIM
  ld (HL), a
  ret
;--- DRAW HEAD --------------------------------------------------------------
DRAW_HEAD:
  ld hl, HEAD_ANIM_TABLE
  ld a, (HEAD_ANIM)
  ld c, a
  ld b, 0
  add hl, bc
  ld a, (HL)
  add a, (GFX_HEAD)
  ld d, a                        ; save GFX_HEAD + HEAD_ANIM to d
   
  ld a, (PR_DIRECTION)           ; check if the direction is
  cp NO_DIRECTION                ; NO_DIRECTION because there are
  jr NZ, NO_NO_DIRECTION         ; no special tiles for this direction,
  ld a, DIRECTION_DOWN           ; so set it to DIRECTION_DOWN
NO_NO_DIRECTION:
  ld b, a                        ; direction * 5, because there are
                                 ; 5 head anims for each direction
  add a, a                       ; * 2
  add a, a                       ; * 4
  add a, b                       ; * 5
  ld b, a
  ld a, d
  add a, b                       ; a is now GFX_HEAD + HEAD_ANIM +
                                 ;          PR_DIRECTION * 5

  push af                        ; save the required tile number
    ld a, (POSITION_Y)
    ld b, a
    ld a, (POSITION_X)
  call CALCULATE_SCREEN_ADDRESS
  pop af                         ; get it back again

  ld b, a                        ; and now draw the tile
  ld d, 0
  call DRAW_TILE
  ret
;--- MOVE LEFT --------------------------------------------------------------
MOVE_LEFT:
  ld a, (POSITION_X)            ; save previous position
  ld (PR_POSITION_X), a
  ld a, (POSITION_Y)
  ld (PR_POSITION_Y), a
  ld hl, POSITION_X             ; get position
  ld a, (HL)
  dec (HL)                      ; move left and save position to a
  dec a
  jp P, NOT_MAX_LEFT
  ld a, MAX_X
  ld (HL), a
NOT_MAX_LEFT:
  push af
  call CLEAR_LAST_TAILPART
  ld hl, LAST_TAILPART        ; save new position to tailpart position
  ld c, (HL)
  ld b, 0
  ld hl, TAILPART_X
  add hl, bc
  pop af
  ld (HL), a
  ld a, (POSITION_Y)            ; save y position to tailpart position
  ld hl, LAST_TAILPART
  ld c, (HL)
  ld b, 0
  ld hl, TAILPART_Y
  add hl, bc
  ld (HL), a
  ret
;--- MOVE RIGHT -------------------------------------------------------------
MOVE_RIGHT:
  ld a, (POSITION_X)            ; save previous position
  ld (PR_POSITION_X), a
  ld a, (POSITION_Y)
  ld (PR_POSITION_Y), a
  ld hl, POSITION_X             ; get position
  ld a, (HL)
  inc (HL)                      ; move right and save position to a
  inc a
  cp MAX_X+1
  jr NZ, NOT_MAX_RIGHT
  ld a, 0
  ld (HL), a
NOT_MAX_RIGHT:
  push af
  call CLEAR_LAST_TAILPART
  ld hl, LAST_TAILPART        ; save new position to tailpart position
  ld c, (HL)
  ld b, 0
  ld hl, TAILPART_X
  add hl, bc
  pop af
  ld (HL), a
  ld a, (POSITION_Y)            ; save y position to tailpart position
  ld hl, LAST_TAILPART
  ld c, (HL)
  ld b, 0
  ld hl, TAILPART_Y
  add hl, bc
  ld (HL), a
  ret
;--- MOVE UP ----------------------------------------------------------------
MOVE_UP:
  ld a, (POSITION_X)            ; save previous position
  ld (PR_POSITION_X), a
  ld a, (POSITION_Y)
  ld (PR_POSITION_Y), a
  ld hl, POSITION_Y             ; get position
  ld a, (HL)
  dec (HL)                      ; move up and save position to a
  dec a
  jp P, NOT_MAX_UP
  ld a, MAX_Y
  ld (HL), a
NOT_MAX_UP:
  push af
  call CLEAR_LAST_TAILPART
  ld hl, LAST_TAILPART        ; save new position to tailpart position
  ld c, (HL)
  ld b, 0
  ld hl, TAILPART_Y
  add hl, bc
  pop af
  ld (HL), a
  ld a, (POSITION_X)            ; save x position to tailpart position
  ld hl, LAST_TAILPART
  ld c, (HL)
  ld b, 0
  ld hl, TAILPART_X
  add hl, bc
  ld (HL), a
  ret
;--- MOVE DOWN --------------------------------------------------------------
MOVE_DOWN:
  ld a, (POSITION_X)            ; save previous position
  ld (PR_POSITION_X), a
  ld a, (POSITION_Y)
  ld (PR_POSITION_Y), a
  ld hl, POSITION_Y             ; get position
  ld a, (HL)
  inc (HL)                      ; move down and save position to a
  inc a
  cp MAX_Y+1
  jr NZ, NOT_MAX_DOWN
  ld a, 0
  ld (HL), a
NOT_MAX_DOWN:
  push af
  call CLEAR_LAST_TAILPART
  ld hl, LAST_TAILPART        ; save new position to tailpart position
  ld c, (HL)
  ld b, 0
  ld hl, TAILPART_Y
  add hl, bc
  pop af
  ld (HL), a
  ld a, (POSITION_X)            ; save x position to tailpart position
  ld hl, LAST_TAILPART
  ld c, (HL)
  ld b, 0
  ld hl, TAILPART_X
  add hl, bc
  ld (HL), a
  ret
;--- CALCULATE SCREEN ADDRESS -----------------------------------------------
; This procedure doesn't check if the x and y positions are out of range.
;--- A = x position   B = y position ----------------------------------------
;--- Output: HL = screen address --------------------------------------------
CALCULATE_SCREEN_ADDRESS:
; x * 2, because you need 2 bytes to define each tile
  add a, a
  ld d, a                       ; save to d
; Get y position
  ld a, b
  ld l, a
  ld h, 0
; y * 64, because one tile line has 64 bytes
  add hl, hl      ; * 2
  add hl, hl      ; * 4
  add hl, hl      ; * 8
  add hl, hl      ; * 16
  add hl, hl      ; * 32
  add hl, hl      ; * 64
; x * 2 + y * 64 + position of left edge
  ld c, d                       ; add x * 2
  ld b, 0
  add hl, bc                    
  ld bc, $78CC                  ; add position of left edge
  add hl, bc                    
  ret
;--- LOAD GAMEGFX -----------------------------------------------------------
LOAD_GAMEGFX:
      ld hl, VRAM_TILES
    call SET_VDP_POINTER
    ld bc, GAMEGFX_SIZE
    ld hl, GAMEGFX
  call LOAD_TO_VRAM
  ret
;--- DRAW LEVEL -------------------------------------------------------------
DRAW_LEVEL:
  ld a, (LEVEL_NUMBER)        ; take the (LEVEL_NUMBER) and
  ld l, a                     ; multiplicate it with 362 because
  ld h, 0                     ; one level has a size of 362 bytes
  add hl, hl                  ; * 2
  ld b, h
  ld c, l
  push bc
  add hl, hl                  ; * 4
  add hl, hl                  ; * 8
  ld b, h
  ld c, l
  push bc
  add hl, hl                  ; * 16
  add hl, hl                  ; * 32
  ld b, h
  ld c, l
  push bc
  add hl, hl                  ; * 64
  ld b, h
  ld c, l
  add hl, hl                  ; * 128
  add hl, hl                  ; * 256
  add hl, bc                  ; * 320
  pop bc
  add hl, bc                  ; * 352
  pop bc
  add hl, bc                  ; * 360
  pop bc
  add hl, bc                  ; * 362

  ld bc, LEVEL_DESCRIPTIONS   ; add the start address of the levels
  add hl, bc                  ; to the level offset
  push hl                     ; save HL
    ld hl, $78CC              ; set vdp pointer to the left edge
  call SET_VDP_POINTER
  pop hl                      ; get back HL
  ld a, (HL)                  ; set start positions from the
  ld (START_X), a             ; first to bytes that describe
  inc hl                      ; the current level
  ld a, (HL)
  ld (START_Y), a
  inc hl

  ld a, l                     ; load the level address into IX
  ld (DUMMY), a
  ld a, h
  ld (DUMMY+1), a
  ld ix, (DUMMY)

  ld d, MAX_X+1               ; set the row counter
  ld e, MAX_Y+1               ; set the line counter
  ld hl, $78CC                ; load the address of the left edge into HL
DRAW_LEVEL_LOOP:
  ld a, (IX)                  ; load tile number and
  add a, GFX_WALLS            ; add the number of the first level tile
  out ($BE), a
  ld a, 0                     ; the tile attribute is always 0
  out ($BE), a
  inc ix                      ; next tile
  dec d                       ; decrease row counter
  jr NZ, DRAW_LEVEL_LOOP

  ld d, MAX_X+1               ; reset the row counter
    ld bc, 64                 ; next line
    add hl, bc
  call SET_VDP_POINTER

  dec e                       ; decrease the line counter
  jr NZ, DRAW_LEVEL_LOOP
  ret





;============================================================================
;=                                                                          =
;= L O G O                                                                  =
;=                                                                          =
;============================================================================
LOGO:
      ld hl, VRAM_TILES+$20                       ; load the
    call SET_VDP_POINTER                          ; required gfx
    ld bc, KONRAD_LOGO_XSIZE*KONRAD_LOGO_YSIZE*32
    ld hl, KONRAD_LOGO
  call LOAD_TO_VRAM
      ld hl, KONRAD_LOGO_POSITION     ; draw the logo 
    call SET_VDP_POINTER
    ld a, 1
    ld b, KONRAD_LOGO_XSIZE
    ld c, KONRAD_LOGO_YSIZE
    ld d, 0
  call DRAW_IMAGE
  call CLEAR_PALETTE                  ; clear the palette
  call ENABLE_SCREEN
    ld a, -6                          ; fade in  
    ld d, 7
    ld e, 3
    ld ix, LOGO_PALETTE
  call FADE_IN
  call GETCH                          ; wait until any button
                                      ; is pressed
    ld a, -1                          ; fade out
    ld d, 6
    ld e, 3
    ld ix, LOGO_PALETTE
  call FADE_OUT
  call DISABLE_SCREEN
  ret

;============================================================================
;=                                                                          =
;= T I T L E                                                                =
;=                                                                          =
;============================================================================
TITLE:
  call CLEAR_TILEMAP              ; clear the tilemap
  call CLEAR_PALETTE
      ld hl, VRAM_TILES+$20       ; load TITLE_GFX
    call SET_VDP_POINTER
    ld bc, TITLE_SIZE
    ld hl, TITLE_GFX
  call LOAD_TO_VRAM
      ld hl, TITLE_POSITION       ; draw TITLE_GFX
    call SET_VDP_POINTER
    ld a, 1
    ld b, TITLE_XSIZE
    ld c, TITLE_YSIZE
    ld d, 0
  call DRAW_IMAGE

    ld a, 42                      ; set the y scroll position
    ld b, $89
  call SET_VDP_REG

  call ENABLE_SCREEN
  ld d, -6                        ; fade in from -6 (very dark)
  ld c, 41                        ; scroll up from y position 41
  ld e, 7                         ; 7 light up steps
SHOW_TITLE_LOOP:
  push de
  push bc
    ld a, d                       ; write the brightness into (DUMMY), ...
    ld (DUMMY), a
    ld ix, TITLE_PALETTE
   call CREATE_PALETTE            ; ... create and display the palette
  pop bc
  ld e, 6                         ; scroll 6 pixels up for each fade in
TITLE_SCROLL:
    ld a, c
    ld b, $89
  call SET_VDP_REG
    ld d, 2                       ; wait 2 VWAITs
  call VWAIT
  dec c                           ; decrease y scroll position
  dec e                           ; next scroll step
  jr NZ, TITLE_SCROLL

  pop de
  inc d                           ; light up
  dec e                           ; next light up step
  jr NZ, SHOW_TITLE_LOOP

  call GETCH                      ; wait until any button is pressed

    ld a, -1                      ; fade out
    ld d, 6
    ld e, 4
    ld ix, TITLE_PALETTE
  call FADE_OUT
  call DISABLE_SCREEN
  ret

;============================================================================
;=                                                                          =
;= T H E   E N D                                                            =
;=                                                                          =
;============================================================================
THE_END:
  call CLEAR_TILEMAP              ; clear the tilemap
  call CLEAR_TILES
  call CLEAR_PALETTE
      ld hl, VRAM_TILES+$20       ; load THE_END_GFX
    call SET_VDP_POINTER
    ld bc, THE_END_SIZE
    ld hl, THE_END_GFX
  call LOAD_TO_VRAM
      ld hl, VRAM_TILES+THE_END_SIZE+$20           ; load TITLE_GFX
    call SET_VDP_POINTER
    ld bc, TITLE_SIZE
    ld hl, TITLE_GFX
  call LOAD_TO_VRAM
      ld hl, THE_END_POSITION     ; draw THE_END_GFX
    call SET_VDP_POINTER
    ld a, 1
    ld b, THE_END_XSIZE
    ld c, THE_END_YSIZE
    ld d, 0
  call DRAW_IMAGE
      ld hl, THE_END_POSITION+256 ; draw TITLE_GFX
    call SET_VDP_POINTER
    ld a, 34
    ld b, TITLE_XSIZE
    ld c, TITLE_YSIZE
    ld d, 0
  call DRAW_IMAGE

  call ENABLE_SCREEN
    ld a, -6                      ; fade in
    ld d, 7
    ld e, 9
    ld ix, TITLE_PALETTE          ; THE_END uses the TITLE_PALETTE, too
  call FADE_IN

  call GETCH                      ; wait until any button is pressed

    ld a, -1                      ; fade out
    ld d, 6
    ld e, 9
    ld ix, TITLE_PALETTE
  call FADE_OUT
  call DISABLE_SCREEN

  jp $0000                        ; reset
  ret

;============================================================================
;=                                                                          =
;= O P T I O N S                                                            =
;=                                                                          =
;============================================================================
OPTIONS:
  call CLEAR_TILEMAP                ; clear tilemap
      ld hl, VRAM_TILES             ; load OPTIONS_GFX
    call SET_VDP_POINTER
    ld bc, OPTIONS_GFX_SIZE
    ld hl, OPTIONS_GFX
  call LOAD_TO_VRAM
  call CLEAR_PALETTE                ; clear the palette

    ld b, 32                        ; Draw "SPEED" border
    ld c, 9
    ld d, 0
    ld hl, $790E+196
  call DRAW_HORZ_MAPLINE
    ld b, 31
    ld c, 9
    ld d, 0
    ld hl, $790E+324
  call DRAW_HORZ_MAPLINE
    ld b, 33
    ld d, 0
    ld hl, $790E+258
  call DRAW_TILE
    ld b, 34
    ld d, 0
    ld hl, $790E+278
  call DRAW_TILE

    ld b, 27                        ; Draw "SPEED" edges
    ld d, 0
    ld hl, $790E+194
  call DRAW_TILE
    ld b, 29
    ld d, 0
    ld hl, $790E+322
  call DRAW_TILE
    ld b, 28
    ld d, 0
    ld hl, $790E+214
  call DRAW_TILE
    ld b, 30
    ld d, 0
    ld hl, $790E+342
  call DRAW_TILE

    ld b, 35                        ; Fill "SPEED" 
    ld d, 0
    ld hl, $790E+262
  call DRAW_TILE
    ld d, 0
    ld hl, $790E+274
  call DRAW_TILE

      ld hl, $790E+264              ; Draw "SPEED" 
    call SET_VDP_POINTER
    ld a, 2
    ld b, 5
    ld c, 1
    ld d, 0
  call DRAW_IMAGE

    ld b, 1                         ; Draw "SPEED" shadows
    ld d, 0
    ld hl, $790E+280
  call DRAW_TILE
    ld b, 1
    ld d, 0
    ld hl, $790E+344
  call DRAW_TILE
    ld c, 11
    ld d, 0
    ld hl, $790E+388
  call DRAW_HORZ_MAPLINE

    ld b, 32                        ; Draw "STARTLEVEL" border
    ld c, 13
    ld d, 0
    ld hl, $790E+580
  call DRAW_HORZ_MAPLINE
    ld b, 31
    ld c, 13
    ld d, 0
    ld hl, $790E+708
  call DRAW_HORZ_MAPLINE
    ld b, 33
    ld d, 0
    ld hl, $790E+642
  call DRAW_TILE
    ld b, 34
    ld d, 0
    ld hl, $790E+670
  call DRAW_TILE

    ld b, 27                        ; Draw "STARTLEVEL" edges
    ld d, 0
    ld hl, $790E+578
  call DRAW_TILE
    ld b, 29
    ld d, 0
    ld hl, $790E+706
  call DRAW_TILE
    ld b, 28
    ld d, 0
    ld hl, $790E+606
  call DRAW_TILE
    ld b, 30
    ld d, 0
    ld hl, $790E+734
  call DRAW_TILE

    ld b, 35                        ; Fill "STARTLEVEL"
    ld d, 0
    ld hl, $790E+646
  call DRAW_TILE
    ld d, 0
    ld hl, $790E+666
  call DRAW_TILE

      ld hl, $790E+648              ; Draw "STARTLEVEL"
    call SET_VDP_POINTER
    ld a, 7
    ld b, 9
    ld c, 1
    ld d, 0
  call DRAW_IMAGE

    ld b, 1                         ; Draw "STARTLEVEL" shadows
    ld d, 0
    ld hl, $790E+672
  call DRAW_TILE
    ld b, 1
    ld d, 0
    ld hl, $790E+736
  call DRAW_TILE
    ld c, 15
    ld d, 0
    ld hl, $790E+772
  call DRAW_HORZ_MAPLINE

    ld ix, OPTIONS_GFX              ; Copy the 2 first tiles
    ld hl, DUMMY_TILES              ; into RAM for scrolling
    ld c, 64
  call COPY_MEM

; Reset the required variables
  ld a, 0
  ld (DUMMY2), a              ; used for the option pointer position
  ld (OLD_JOY_STATUS), a
  ld (STARTLEVEL), a
  ld (DUMMY3), a              ; used for slowing down the fading
  ld a, MINIMAL_SPEED+5
  ld (SPEED), a
  ld a, -7
  ld (DUMMY), a               ; used for the fading brightness value

  call UPDATE_OPTION_SCREEN

  call ENABLE_SCREEN
OPTIONS_LOOP:
    ld d, 1
  call VWAIT

    ld ix, OPTIONS_PALETTE
    ld b, 7
  call FADE_IN_ONE_STEP

  ld hl, DUMMY_TILES            ; Scroll the 2 first tiles
  ld c, 64                      ; 2 tiles * 32 bytes => 64 bytes
OPTIONS_SCROLL_LOOP:
  rlc (HL)                      ; scroll each bit to the left
  inc hl
  dec c
  jr NZ, OPTIONS_SCROLL_LOOP

      ld hl, VRAM_TILES         ; Copy the 2 first tiles from RAM into VRAM
    call SET_VDP_POINTER
    ld bc, 64                   ; 2 tiles * 32 bytes => 64 bytes
    ld hl, DUMMY_TILES
  call LOAD_TO_VRAM

;............................................................................
; Move the pointer and change the options
  in a, (P1_CONTROL_PORT)
  ld e, a
;............................................................................
  and P1_DOWN
  jr NZ, OP_NOT_DOWN
  ld a, (OLD_JOY_STATUS)          ; don't go down again if
  and P1_DOWN                     ; this was already done previously
  jr Z, OP_NOT_DOWN

  ld a, (DUMMY2)                  ; get the option pointer position
  ld c, a
  ld a, 1                         ;     option pointer position =
  sub c                           ; 1 - option pointer position
  ld (DUMMY2), a
OP_NOT_DOWN:
;............................................................................
  ld a, e
  and P1_UP                       
  jr NZ, OP_NOT_UP
  ld a, (OLD_JOY_STATUS)          ; don't go up again if
  and P1_UP                       ; this was already done previously
  jr Z, OP_NOT_UP

  ld a, (DUMMY2)                  ; get the option pointer position
  ld c, a
  ld a, 1                         ;     option pointer position =
  sub c                           ; 1 - option pointer position
  ld (DUMMY2), a
OP_NOT_UP:
;............................................................................
  ld a, e
  and P1_RIGHT
  jr NZ, OP_NOT_RIGHT
  ld a, (OLD_JOY_STATUS)          ; don't go right again if
  and P1_RIGHT                    ; this was already done previously
  jr Z, OP_NOT_RIGHT

  ld a, (DUMMY2)
  cp 1                            ; change the startlevel if the
  jr Z, INCREASE_STARTLEVEL       ; option pointer position is 1

  ld a, (SPEED)
  dec a                           ; the speed has to be decreased because
  ld (SPEED), a                   ; it is actually a slowdown value
  cp MINIMAL_SPEED - 1            ; check if it's out of range
  jr NZ, OP_NOT_RIGHT
  ld a, 9 + MINIMAL_SPEED
  ld (SPEED), a
  jr OP_NOT_RIGHT

INCREASE_STARTLEVEL:
  ld a, (STARTLEVEL)
  inc a
  ld (STARTLEVEL), a
  cp 10
  jr NZ, OP_NOT_LEFT
  ld a, 0
  ld (STARTLEVEL), a

OP_NOT_RIGHT:
;............................................................................
  ld a, e
  and P1_LEFT
  jr NZ, OP_NOT_LEFT
  ld a, (OLD_JOY_STATUS)          ; don't go left again if
  and P1_LEFT                     ; this was already done previously
  jr Z, OP_NOT_LEFT

  ld a, (DUMMY2)
  cp 1                            ; change the startlevel if the
  jr Z, DECREASE_STARTLEVEL       ; option pointer position is 1

  ld a, (SPEED)
  inc a                           ; the speed has to be increased because
  ld (SPEED), a                   ; it is actually a slowdown value
  cp 10 + MINIMAL_SPEED           ; check if it's out of range
  jr NZ, OP_NOT_LEFT
  ld a, MINIMAL_SPEED
  ld (SPEED), a
  jr OP_NOT_LEFT

DECREASE_STARTLEVEL:
  ld a, (STARTLEVEL)
  dec a
  ld (STARTLEVEL), a
  cp 255
  jr NZ, OP_NOT_LEFT
  ld a, 9
  ld (STARTLEVEL), a

OP_NOT_LEFT:
;............................................................................

; Check the start button
  in a, (GG_START_BUTTON_PORT)  ; check if the start button is pressed
  and GG_START_BUTTON
  jr Z, QUIT_OPTIONS            ; return if yes

  ld a, e                       ; save the joystick status
  ld (OLD_JOY_STATUS), a        ; of the last frame

  call UPDATE_OPTION_SCREEN

  jp OPTIONS_LOOP               ; jump back
  ret
QUIT_OPTIONS:
  call DISABLE_SCREEN
  ret

;--- UPDATE OPTION SCREEN ---------------------------------------------------
UPDATE_OPTION_SCREEN:
  push bc                        ; save everything that is important 
  push de

    ld b, 35                     ; First clear the option pointer on both possible positions
    ld d, 0
    ld hl, $790E+260
  call DRAW_TILE
    ld d, 0
    ld hl, $790E+644
  call DRAW_TILE

  ld b, 16                       ; Not draw the option pointer at the current
  ld a, (DUMMY2)                 ; option pointer position
  cp 1
  jr Z, OP_SECOND_POSITION
    ld d, 0
    ld hl, $790E+260
  call DRAW_TILE
  jr UPDATE_OPTION_SCREEN_2
OP_SECOND_POSITION:
    ld d, 0
    ld hl, $790E+644
  call DRAW_TILE

UPDATE_OPTION_SCREEN_2:          ; Draw the numbers
    ld a, (SPEED)
    ld b, a
    ld a, 26 + MINIMAL_SPEED
    sub b                        ; subtract the speed because
    ld b, a                      ; it is a slowdown value
    ld d, 0
    ld hl, $790E+276
  call DRAW_TILE
    ld a, (STARTLEVEL)
    add a, 17
    ld b, a
    ld d, 0
    ld hl, $790E+668
  call DRAW_TILE

  pop de                         ; get everything back that is important 
  pop bc                       
  ret

;============================================================================
;=                                                                          =
;= S U B F U N C T I O N S                                                  =
;=                                                                          =
;============================================================================

.include palette.inc
.include memory.inc
.include vdp.inc
.include drawing.inc

;--- WAIT FOR NEXT VBLANK ---------------------------------------------------
;--- D = how many times to wait ---------------------------------------------
VWAIT:
  in a,($BF)
  and $80
  jr Z, VWAIT
  dec d
  jr NZ, VWAIT
  ret
;--- WAIT FOR PRESSING ANY BUTTON -------------------------------------------
GETCH:
  in a, (GG_START_BUTTON_PORT)        ; don't continue if any
   and GG_START_BUTTON                ; is already pressed
    jr Z, GETCH
  in a, (P1_CONTROL_PORT)
  ld b, a
   and P1_B1
    jr Z, GETCH
  ld a, b
   and P1_B2
    jr Z, GETCH
GETCH_LOOP:                           ; wait until a button is pressed
  ld hl, RANDOM                       ; change the random numbers
  inc (HL)
  ld hl, RANDOM2
  inc (HL)
  ld hl, RANDOM3
  inc (HL)

  in a, (GG_START_BUTTON_PORT)
   and GG_START_BUTTON
    jr Z, GETCH_EXIT
  in a, (P1_CONTROL_PORT)
  ld b, a
   and P1_B1
    jr Z, GETCH_EXIT
  ld a, b
   and P1_B2
    jr Z, GETCH_EXIT
  jr GETCH_LOOP
GETCH_EXIT:
  ret
;--- WAIT FOR PRESSING START ------------------------------------------------
; This procedure is the same as the one above but
; only handles the start button (used for pause).
GETST:
  in a, (GG_START_BUTTON_PORT)
   and GG_START_BUTTON
    jr Z, GETST
GETST_LOOP:
  ld hl, RANDOM                       ; change the random numbers
  inc (HL)
  ld hl, RANDOM2
  inc (HL)
  ld hl, RANDOM3
  inc (HL)

  in a, (GG_START_BUTTON_PORT)
   and GG_START_BUTTON
    jr Z, GETST_EXIT
  jr GETST_LOOP
GETST_EXIT:
  ret
;--- SETUP DEFAULT FRAMES VALUES --------------------------------------------
; (only the first 16k are used)
SETUP_FRAMES:
  ld a, 0
  ld ($FFFC), a
  ld ($FFFD), a
  ld a, 1
  ld ($FFFE), a
  ld ($FFFF), a
  ret
;--- LOAD DATA TO VRAM ------------------------------------------------------
;--- BC = length of data chunk   HL = address of data chunk -----------------
;--- Requires VDP Pointer to be set -----------------------------------------
LOAD_TO_VRAM:
  ld a, (HL)
  out ($BE), a
  inc hl
  dec bc
  ld a, b
  or c
  jr NZ, LOAD_TO_VRAM
  ret
;--- DISABLE SPRITES --------------------------------------------------------
; This procedure writes the STOP_SPRITES-code into
; the y position of sprite specified in register A.
;--- A = number of sprite ---------------------------------------------------
DISABLE_SPRITES:
  ld hl, VRAM_SPRITES
  ld b, 0
  ld c, a
  add hl, bc
  call SET_VDP_POINTER
  ld a, STOP_SPRITES
  out ($BE), a
  ret
;--- CLEAR TILES ------------------------------------------------------------
CLEAR_TILES:
    ld hl, VRAM_TILES
  call SET_VDP_POINTER
  ld bc, $E00   ; all tile definitions together
                ; have a size of $E00 * 4 bytes = $3800 bytes
CLEAR_LOOP:
  ld a, $00
  out ($be), a  ; write 4 times (faster than only one write per loop)
  out ($be), a
  out ($be), a
  out ($be), a
  inc hl
  dec bc
  ld a, b
  or c
  jr NZ, CLEAR_LOOP
  ret
;--- CLEAR TILEMAP ----------------------------------------------------------
CLEAR_TILEMAP:
  ld hl, VRAM_TILEMAP
  call SET_VDP_POINTER
  ld bc, $1C0   ; the tilemap has a size of $1C0 * 4 bytes = $700 bytes
CLEAR_LOOP2:
  ld a, $00
  out ($be), a  ; write 4 times (faster than only one write per loop)
  out ($be), a
  out ($be), a
  out ($be), a
  dec bc
  ld a, b
  or c
  jr NZ, CLEAR_LOOP2
  ret
;--- ENABLE SCREEN ----------------------------------------------------------
; This procedure sets bit 6 of VDP register 1.
; But it doesn't care of other bits.
ENABLE_SCREEN:
  ld a, $40
  ld b, $81
  call SET_VDP_REG
  ret
;--- DISABLE SCREEN ---------------------------------------------------------
; This procedure clears bit 6 of VDP register 1.
; But it doesn't care of other bits.
DISABLE_SCREEN:
  ld a, $00
  ld b, $81
  call SET_VDP_REG
  ret
;--- SET SPRITE -------------------------------------------------------------
;--- A = tile number   C = sprite number   D = x position    E = y position -
SET_SPRITE:
  push af
    ld b, 0                  ; clear the high byte of BC
    ld hl, VRAM_SPRITES
    add hl, bc               ; => HL = VRAM_SPRITES + sprite number
    push hl                  ; save it for other writes
  call SET_VDP_POINTER
    ld a, e                  ; set the y position
  out ($BE), a

    pop hl                   ; get HL back
    add hl, bc               ; add sprite number again
    ld bc, 128
    add hl, bc               ; => HL = VRAM_SPRITES + sprite number * 2 + 128
  call SET_VDP_POINTER
    ld a, d
  out ($BE), a               ; set the x position
    pop af
  out ($BE), a               ; set the tile number
  ret





;============================================================================
;=                                                                          =
;= D A T A                                                                  =
;=                                                                          =
;============================================================================

LEVEL_DESCRIPTIONS:
.include levels.dat

.org $4000     ; Frame 1 starts here

; Palettes
LOGO_PALETTE:
.include logopal
  .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0   ; empty sprite palette
  .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
TITLE_PALETTE:
.include titlepal
  .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0   ; empty sprite palette
  .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
OPTIONS_PALETTE:
.include optpal
  .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0   ; empty sprite palette
  .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
GAME_PALETTE:
.include gamepal                        ; both tiles and sprites use
.include gamepal                        ; the same palette in the game

; GFX:
KONRAD_LOGO:
.include logogfx
TITLE_GFX:
.include titlegfx
OPTIONS_GFX:
.include optgfx
GAMEGFX:
.include gamegfx
THE_END_GFX:
.include endgfx

HEAD_ANIM_TABLE:
   .db 0,0,0,1,1,1
   .db 2,2,2,2,1,1
   .db 1,0,0,0,3,3
   .db 3,4,4,4,4,3
   .db 3,3

VDP_REGS_TABLE:
   .db $06, $00, $FF, $FF, $FF, $FF, $00, $00, $00, $00, $FF, $00, $00, $00, $00, $00

.org $8000
.end
